#####################################################################################################################
# Licensed Materials - Property of IBM
# Rational ClearQuest
# (C) Copyright IBM Corp. 2003-2011.  All Rights Reserved.
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
# 
#
# Updated by: Shirley Dango
# Updated on: April 21, 2003
# Updated January, 2008 by Katie Carpenter
######################################################################################################################
# 
#This script utilized the ClearQuest COM API and the ClearCase cleartool commands.
# 
# Variable used in this script are as followings:
#	user		The login id of the ClearQuest user
#	pw			The password of the ClearQuest user
#	dbname		The schema repository connection to the database
#	dbset		The user database connection
#	baseline	The name of the baseline record
#	pvob		The name of the project vob for this baseline
#	get_promotionlevel	The subrountine finds the promotion level for the baseline
#	logfile			The path where the log file will be sent
#	test			This is a flag that can be set for testing file inputs
#	help			This is a flag for the help
#	EDIT_BUILD_ACTION	This variable will set a session variable for ClearQuest access.
#######################################################################################################################
#
# Change Log:
# Flag      Item                Date        Programmer         Comments
# thl1   RATLC01087702       06/28/2005      THL               UCM Integration Scripts should use ratlperl not cqperl
# thl2   RATLC00475437       10/18/2005      THL               the baseline create perl module needs to also check the returned 
#                                                              status from CQ API calls.
# thl3   RATLC00475646       10/19/2005      THL               Need to check for \ on pvob parm
#        RATLC00396394	     1/10/2008       katherns          Update obsolete field
#
#######################################################################################################################
#!ratlperl -l -w
# Changed above line to read ratlperl from cqperl thl1
use strict; 
use vars qw($Bin);
use FindBin qw($Bin);
use lib "$Bin";
use Getopt::Long;
use File::Basename;
use Carp;
use FileHandle;
use File::Spec;  
use CQPerlExt;
use Env;
use LWP::Simple;
use POSIX;
my $promotion_level;
my $slashvob;
my $noslashvob;
my ($entity);
my ($failure);
my ($status,$resultSetStatus); #thl2
my $progname = basename $0;
use subs qw(get_promotionlevel main);

# Unbuffer STDERR and STDOUT
select STDERR;  
$| = 1;
select STDOUT;
$| = 1;

#Define Logger parameters
use Logger qw(CONSOLE_ON CONSOLE_OFF CONSOLE_TERSE);
my $logger = Logger->new(); # the logger element

# Create the utility object
use UtilLib;
my $ut = UtilLib->new();
my $log_time = $ut->ut_file_name_date_time_stamp();

my $progname = basename $0;

my $result = main();

if ($result) {
    exit(0); # DOS Success Code
} else {
    exit(1); # DOS Error Code
}

#####
# Subroutine to display the usage
sub Usage {
    print "

    ʹ CQ API ¼Ựʵó
    ÷
         $progname -user <û>
         -pw <>
         -dbname <ݿ>
         -dbset <ݿ⼯>
         -pvob <pvob>
         -baseline <>
		 -logfile <־ļ>
		 -test
		 -help
";
return 0;
}

sub main {
####
# Define the input parameters
my @opts = qw(
    user=s		
    pw=s		
    dbname=s		
    dbset=s		
    pvob=s		
    baseline=s	
    logfile=s
    test
    help
);

my %opts = (
    user => undef,
    pw => undef,
    dbname => undef,
    dbset => undef,
    pvob => undef,
    baseline => undef,
    logfile => undef,
);


####
# Get the input parameters
unless (GetOptions(\%opts, @opts)) {
     	$logger->log_print_message("ʾ÷");	
      	my $retval = &Usage;
      	return 0;
}

#Define the LogFile
if ($opts{logfile}) {
	my $retval;
	my $logdir = dirname($opts{logfile});
	if ( ! -e $logdir) {
		$retval = $ut->ut_create_directory_tree($logdir);
		if ( ! $retval) {
            		$logger->log_print_error( __LINE__ . "޷־ļĿ¼$logdir");
            		$logger->log_print_error( __LINE__ . "űڲĿ¼¼ִС");
        	}
	}
	$retval = $logger->log_set_file_on($opts{logfile});
	if ( ! $retval) {
        	$logger->log_print_error( __LINE__ . "޷־ļ$opts{logfile}");
        	$logger->log_print_error( __LINE__ . "űڲĿ¼¼ִС");
   	 }
	 $logger->log_set_console(CONSOLE_OFF);
	$logger->log_print_message("¼ $log_time ʼ"); 
}	 



####
# Display the help usage
if ($opts{help}) {
	print "-user		          ClearQuest ûĵ¼ʶ\n";
	print "-pw		            ClearQuest û롣\n";
	print "-dbname		        ݿģʽ洢ӡ\n";
	print "-dbset		         ûݿӡ\n";
	print "-pvob		          Ŀ VOB ơ\n";
	print "-baseline	       ߵơ\n";
	print "-logfile		       ־ļ͵·\n";
	print "-test		          Ϊļõı־\n";
	print "-help		          ڰı־\n";
    exit;
}
# Display test inputs to log file 
if ($opts{test}){
	$logger->log_print_message("main: Ѷû룺");
	$logger->log_print_message("main: Ѷû¼Ϊ $opts{user}");
	$logger->log_print_message("main: Ѷݿ⼯Ϊ $opts{dbset}");
	$logger->log_print_message("main: ѶݿΪ $opts{dbname}");
	$logger->log_print_message("main: ѶĻΪ $opts{baseline}");
	$logger->log_print_message("main: ѶĿ VOB Ϊ $opts{pvob}");
	if ($opts{logfile}){ $logger->log_print_message("Ѷ־ļΪ $opts{logfile}");}
	return 1;
}

####
# End program if any of the parameters are not defined
if (!defined $opts{user}
  or !defined $opts{pw}
  or !defined $opts{dbname}
  or !defined $opts{dbset}
  or !defined $opts{pvob}
  or !defined $opts{baseline}) {
  $logger->log_print_message("ʾ÷");
  my $retval = &Usage();
  return 0;
}
;


####
#### Start a CQ session
my $session = CQPerlExt::CQSession_Build();
# Set session variable 
#$session->SetNameValue("EDIT_BUILD_ACTION", "ACCESS_OK");
#### Logon to the specified database

eval {$session->UserLogon("$opts{user}", "$opts{pw}", "$opts{dbname}", "$opts{dbset}")};

if ($@) {
	$logger->log_print_error(__LINE__ . "main: ¼ $opts{dbname} ʧܣ$@"); 
	return 0;
}

$logger->log_print_success("main: ¼ $opts{dbname} ɹ");
#

#### Set up vobs
##Passing in the vob to CQ requires no slashes on the vob, passing in to CC requires a slash on the vob
sub slash_noslash {
	my $pvob = shift;
	my $slashvob;
	my $noslashvob;
	if ($pvob !~ /[\/\\]/) {
	  $slashvob = '\\' .$pvob;
	  $noslashvob = $pvob;
	}
	else{
	  $slashvob = $pvob;
	  $noslashvob = $pvob ;
	  $noslashvob=~ s/\\//g; 
	}
	return ($slashvob, $noslashvob);
}
($slashvob, $noslashvob ) = &slash_noslash($opts{pvob});




#### Get record from ClearQuest

#### Build a query to find the Baseline record

my $queryDef = $session->BuildQuery("ALMBaseline");
my $failure = $queryDef->BuildField("Name");

my $operator = $queryDef->BuildFilterOperator($CQPerlExt::CQ_BOOL_OP_AND);
$operator->BuildFilter("Name", $CQPerlExt::CQ_COMP_OP_EQ, ["$opts{baseline}"]);
$operator->BuildFilter("PVOB_OrLocation", $CQPerlExt::CQ_COMP_OP_EQ, ["$noslashvob"]); #021703

my $resultSet;

eval {$resultSet = $session->BuildResultSet($queryDef)};
if ($@) {
	$logger->log_print_error(__LINE__ . "main: ޷ͨѯ"); 
	$logger->log_print_error(__LINE__ . "main: $@"); 
	return 0;
}

$logger->log_print_message("main: ͨѯ");

$resultSet->Execute;

$resultSetStatus = $resultSet->MoveNext;  # thl2
if ($resultSetStatus != $CQPerlExt::CQ_SUCCESS){   #thl2
	$logger->log_print_error(__LINE__ . "main: δҵ߼¼");
	$logger->log_print_error(__LINE__ . "main: ʹ create_baseline_record.pl һµĻ߼¼");
	$logger->log_print_success("main:  update_baseline_record ű");
        CQPerlExt::CQSession_Unbuild($session);
        return 0;
}

#### Print out the resultset
while($resultSetStatus == $CQPerlExt::CQ_SUCCESS) { #thl2
	my $foundrecord = $resultSet->GetColumnValue(1);
	print ("$foundrecord\n");
	$logger->log_print_message("main: ҵ¼$foundrecord");
	
	####Get record 
	my $record1;
	eval {$record1 = $session->GetEntity("ALMBaseline", $noslashvob.' '.$foundrecord)};
	if ($@) {  
		$logger->log_print_error(__LINE__ . "main: ݿδҵѯƥļ¼"); 
		$logger->log_print_error(__LINE__ . "main:  create_baseline_record.pl ԴһµĻ߼¼"); 
                return 0;
	}
	
	## Find the promotion level of the baseline
	$promotion_level = get_promotionlevel(baseline => $opts{baseline}, pvob => $slashvob);#changed
	if (!$promotion_level) {
		$logger->log_print_error(__LINE__ . "main: ޷"); 
		$logger->log_print_error(__LINE__ . "main: ˳δκλ߼¼"); 
	}	
	
	## Edit the record to add the new promotion level
        $status = "" ;                                             #thl2
	eval {$session->EditEntity($record1,"modify")};  
	if ($@) {                                   
		$logger->log_print_error(__LINE__ . "main: ޷޸ļ¼ $record1");  
		$logger->log_print_error(__LINE__ . "main: $@");
                return 0;
	}
	
	my $failure = $record1->SetFieldValue("PromotionLevel", $promotion_level);


	##If baseline is obsolete in CC, it should be obsolete in CQ
	my $cmd = qq(cleartool lslock baseline:"$opts{baseline}\@$slashvob"); 
	my ($retval, $output) = $ut->ut_shell($cmd);
        unless (defined $retval){		
		$logger->log_print_error(__LINE__ ."޷Ϣ");
		$logger->log_print_error(__LINE__ ."޷ʹϢ»");
		
        }
	
	if($output =~ m/locked/i) {

		if($output =~ m/obsolete/i) {
		  my $failure = $record1->SetFieldValue("Obsolete", "Yes");			
	          $logger->log_print_message("ΪObsolete֤");
	        }
		else {
		  $logger->log_print_message("ѱ");
		}
	  
	}
	else {
		my $failure = $record1->SetFieldValue("Obsolete", "No");		
		$logger->log_print_message("δΪObsolete֤");
	}

        
	#Validate the Baseline Record
	$logger->log_print_message("main: ֤߼¼");
        $status = "" ;                                             #thl2
	eval {$status = $record1->Validate()};
	if (($@) || ($status)) {                                   #thl2
		$logger->log_print_error(__LINE__ . "main: ޷֤¼ $record1$@"); #thl2
                $logger->log_print_error(__LINE__ . "ص״̬Ϊ $status") ;  #thl2
		return 0;
	}
	$status = "" ; #thl2
	
	# Committing the Baseline Record
	$logger->log_print_message("main: ʵ߼¼");
	eval {$status = $record1->Commit()}; #thl2
	if (($@) || ($status)) {                                   #thl2
		$logger->log_print_error(__LINE__ . "main: ޷ʵ¼ $record1$@"); #thl2
                $logger->log_print_error(__LINE__ . "ص״̬Ϊ $status") ;  #thl2
		return 0;
	}
	$record1 = undef;

	$logger->log_print_success("main: ߼¼Ѹ");
	CQPerlExt::CQSession_Unbuild($session);
	return 1;	
	$resultSet->MoveNext ;  
} # end of loop

# Write to log file 
$logger->log_print_success("main:  update_baseline_record ű");
CQPerlExt::CQSession_Unbuild($session);
return 1;
}
# Find Promotion Level for the Baseline Record  
#
sub get_promotionlevel { 
	my %opts = @_;
	my $cmd = qq(cleartool lsbl -l "$opts{baseline}\@$slashvob"); 
	my ($retval, $output) = $ut->ut_shell($cmd);
        unless (defined $retval){
		$logger->log_print_error(__LINE__ . "Cleartool $cmd"); 
		$logger->log_print_error(__LINE__ . "޷Ϣ");
		$logger->log_print_error(__LINE__ . "޷ûϢ´ߡ");
		return 0;
        }
	$output =~ /promotion level: (.*?)$/ms;
	my $plevel = $1;
	chomp $plevel;
	$logger->log_print_success("Ϊ $plevel");
	return $plevel;
} 
# end of script
